import os
import random
import string
try:
	from typing import Dict, Optional, Tuple, Union
except ImportError:
	pass

DEFAULT_CONTEXT = os.environ.get("OX/CONTEXT/ID", 10)
DOMAIN = os.environ.get("DOMAINNAME", "localhost.localdomain")
LOCAL_TIMEZONE = os.environ.get("OX/TIMEZONE", "Europe/Berlin")
OX_MASTER_ADMIN = os.environ.get("OX/ADMINMASTER/USER", "oxadminmaster")
OX_MASTER_PASSWORD = os.environ.get("OX/ADMINMASTER/PASSWORD")
QUOTA = os.environ.get("OX/CONTEXT/DEFAULT_QUOTA", 32697)
SECRETS_DIR = "/etc/ox-secrets"
OX_SOAP_SERVER = os.environ.get("OX/SOAP/SERVER", "127.0.0.1")


class NoContextAdminPassword(Exception):
	pass


def get_new_context_attributes(context_id):  # type: (int) -> Dict[str, str]
	name = 'context{}'.format(context_id)
	mapping = ['context{}'.format(context_id), str(context_id)]
	return {
		'id': context_id,
		'name': name,
		'mapping': mapping,
		'timezone': LOCAL_TIMEZONE,
		'username': get_context_admin_user(context_id),
		'displayname': 'OX Admin',
		'givenname': 'OX',
		'surname': 'Admin',
		'email': '{}@{}'.format(get_context_admin_user(context_id), DOMAIN),
		'quota': QUOTA,
		'password': get_random_password(),
	}


def get_random_password(length=15):  # type: (Optional[int]) -> str
	pw = list()
	if length >= 4:
		pw.append(random.choice(string.ascii_lowercase))
		pw.append(random.choice(string.ascii_uppercase))
		pw.append(random.choice(string.digits))
		pw.append(random.choice(string.punctuation))
		length -= len(pw)
	pw.extend(random.choice(string.ascii_letters + string.digits + string.punctuation) for _x in range(length))
	random.shuffle(pw)
	return ''.join(pw)


def get_context_admin_user(context_id):  # type: (Union[int, str]) -> str
	if int(context_id) == DEFAULT_CONTEXT:
		return 'oxadmin'
	else:
		return 'oxadmin-context{}'.format(context_id)


def save_context_admin_password(context_id, password, secrets_dir=SECRETS_DIR):
	# type: (Union[int, str], str, Optional[str]) -> None
	context_admin_pwd_file = os.path.join(secrets_dir, 'context{}.secret'.format(context_id))
	open(context_admin_pwd_file, 'w')
	os.chmod(context_admin_pwd_file, 0o600)
	with open(context_admin_pwd_file, 'w') as fd:
		fd.write(password)


def remove_context_admin_password(context_id, secrets_dir=SECRETS_DIR):
	# type: (Union[int, str], Optional[str]) -> None
	context_admin_pwd_file = os.path.join(secrets_dir, 'context{}.secret'.format(context_id))
	if os.path.exists(context_admin_pwd_file):
		os.unlink(context_admin_pwd_file)


def get_credentials_for_context(context_id):  # type: (Union[int, str]) -> Tuple[str, str]
	admuser = get_context_admin_user(context_id)
	admpass = get_context_admin_password(context_id)
	return admuser, admpass


def get_context_admin_password(context_id, secrets_dir=SECRETS_DIR):
	# type: (Union[int, str], Optional[str]) -> str
	context_admin_pwd_file = os.path.join(secrets_dir, 'context{}.secret'.format(context_id))
	try:
		with open(context_admin_pwd_file) as fd:
			return fd.read().strip()
	except IOError:
		raise NoContextAdminPassword(context_id, context_admin_pwd_file)


def get_master_credentials(secrets_dir=SECRETS_DIR):  # type: (Optional[str]) -> Tuple[str, str]
	global OX_MASTER_PASSWORD
	if not OX_MASTER_PASSWORD:
		ox_master_secret_path = os.path.join(secrets_dir, "master.secret")
		with open(ox_master_secret_path, "r") as fp:
			OX_MASTER_PASSWORD = fp.read().strip()
	return OX_MASTER_ADMIN, OX_MASTER_PASSWORD


def set_secrets_dir(secrets_dir):  # type: (str) -> None
	global SECRETS_DIR
	SECRETS_DIR = secrets_dir


def set_soap_server(server):  # type: (str) -> None
	global OX_SOAP_SERVER
	OX_SOAP_SERVER = server
